/************************************************************
*
* REXLANG - Reading data from MCP23017 port expander - 8 output(bank A) & 8 input (bank B)
*
*************************************************************/


string input(14) i2c_bus;
long output(0) digital_in;
long input(15) i2c_chip_address;
long input(0) digital_out;
double output(1) info;

//declaration of variables
long i2c_bufTx[3]; //buffer for transmitting data
long i2c_bufRx[3]; //buffer for receiving data
long i2c_bus_handle;
long i2c_write_count;
long i2c_read_count;
long i2c_ret_fun;

//the init procedure is executed once when the REXLANG function block initializes
int init(void)
{
  i2c_bus_handle = OpenI2C(i2c_bus); // open I2C bus
  info = i2c_bus_handle;
  //Setting PORTA to output
  i2c_bufTx[0] = 0x00; //register no. (IODIRA) 
  i2c_bufTx[1] = 0x00; //IODIRA data, bitmask, 0=output, 1=input
  i2c_write_count = 2;
  i2c_read_count = 0;
  i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);
  //Setting PORTB to input
  i2c_bufTx[0] = 0x01; //register no. (IODIRB) 
  i2c_bufTx[1] = 0xFF; //IODIRB data, bitmask, 0=output, 1=input
  i2c_write_count = 2;
  i2c_read_count = 0;
  i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);
  //Enabling pull-up resistors on PORTB
  i2c_bufTx[0] = 0x0D; //register no. (GPPUB) 
  i2c_bufTx[1] = 0xFF; //GPPUB data, bitmask, 0=no pull-up, 1=pull-up enabled
  i2c_write_count = 2;
  i2c_read_count = 0;
  i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count); 
  return 0;
}

//the main procedure is executed once in each sampling period
long main(void)
{
  //Controlling outputs
  i2c_bufTx[0] = 0x14; //register no. (OLATA)
  i2c_bufTx[1] = digital_out & 0xFF; //masking the data to control the outputs
  i2c_write_count = 2;
  i2c_read_count = 0;
  i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);
    
  //Reading inputs
  i2c_bufTx[0] = 0x13; //register no. (GPIOB)
  i2c_write_count = 1;
  i2c_read_count = 1;
  i2c_ret_fun = I2C(i2c_bus_handle, i2c_chip_address, i2c_bufTx, i2c_write_count, i2c_bufRx, i2c_read_count);
  digital_in = i2c_bufRx[0]; //publishing the received data
  return 0;
}

//the exit procedure is executed once when the task is correctly terminated
// (system shutdown, downloading new control algorithm, etc.)
long exit(void)
{
  if (i2c_bus_handle>=0) Close(i2c_bus_handle); // close I2C bus
  return 0;
}